# Building LLM apps using Llama on Together AI

This folder contains demos on how to use Llama on [Together AI](https://www.together.ai/) to quickly build LLM apps.

 ## Prerequisites

To make the most of the examples in this cookbook, you'll need a Together AI API key (sign up for free [here](https://api.together.ai/signin)).

While the code examples are primarily written in Python/JS, the concepts can be adapted to any programming language that supports interaction with the Together API.

## Cookbooks

| Cookbook | Description | Open |
| -------- | ----------- | ---- |
| [MultiModal RAG with Nvidia Investor Slide Deck](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/multimodal_RAG_with_nvidia_investor_slide_deck.ipynb) | Multimodal RAG using Nvidia investor slides. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/multimodal_RAG_with_nvidia_investor_slide_deck.ipynb) [![](https://uohmivykqgnnbiouffke.supabase.co/storage/v1/object/public/landingpage/youtubebadge.svg)](https://youtu.be/IluARWPYAUc?si=gG90hqpboQgNOAYG)|
| [Llama Contextual RAG](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/llama_contextual_RAG.ipynb) | Implementation of Contextual Retrieval using Llama models. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/llama_contextual_RAG.ipynb) |
| [Llama PDF to podcast](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/pdf_to_podcast_using_llama_on_together.ipynb) | Generate a podcast from PDF content using Llama. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/pdf_to_podcast_using_llama_on_together.ipynb) |
| [Knowledge Graphs with Structured Outputs](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/knowledge_graphs_with_structured_outputs.ipynb) | Get Llama to generate knowledge graphs. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/knowledge_graphs_with_structured_outputs.ipynb) |
| [Structured Text Extraction from Images](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/structured_text_extraction_from_images.ipynb) | Extract structured text from images. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/structured_text_extraction_from_images.ipynb) |
| [Text RAG](https://github.com/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/text_RAG_using_llama_on_together.ipynb) | Implement text-based Retrieval-Augmented Generation. | [![Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/meta-llama/llama-recipes/blob/main/recipes/3p_integrations/togetherai/text_RAG_using_llama_on_together.ipynb) |

## Explore Further

Looking for more resources to enhance your experience with open source models? Check out these helpful links:

- [Together AI developer documentation](https://docs.together.ai/docs/introduction)
- [Together AI API support docs](https://docs.together.ai/reference/chat-completions-1)
- [Together AI Discord community](https://discord.gg/9Rk6sSeWEG)

## Additional Resources

- [Together AI Research](https://www.together.ai/research): Explore papers and technical blog posts from our research team.
- [Together AI Blog](https://www.together.ai/blog): Explore technical blogs, product announcements and more on our blog.
